/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "gctl/core.h"
#include "gctl/algorithm.h"
#include "../lib/optimization.h"
#include "iostream"
#include "iomanip"

using std::cout;
using std::endl;
using std::setw;

int main(int argc, char const *argv[])
{
	gctl::matrix<double> A(4, 3);
	for (int i = 0; i < A.row_size(); i++)
	{
		for (int j = 0; j < A.col_size(); j++)
		{
			A[i][j] = 3*(i+1) + j - 2;
		}
	}
	A[3][1] = 1;

	cout<<"A(" << A.row_size() << ", " << A.col_size() << ") = " <<endl;
	A.show();

	gctl::svd svdd;
	svdd.decompose(A);

	cout<<"U(" << svdd.U.row_size() << ", " << svdd.U.col_size() << ") = " <<endl;
	svdd.U.show();

	cout<<"S(" << svdd.S.size() << ") = " << endl;
	svdd.S.show();

	cout<<"V(" << svdd.V.row_size() << ", " << svdd.V.col_size() << ") = " <<endl;
	svdd.V.show();

	int sig_num = svdd.get_singular_number();
	double tmp_d;
	gctl::array<double> tmp(sig_num);

	cout<<"U^T * S * V(" << A.row_size() << ", " << A.col_size() << ") = " <<endl;
	for(int i=0;i<A.row_size();i++)
	{
		for (int k = 0; k < sig_num; k++)
		{
			tmp[k] = svdd.U[k][i] * svdd.S[k];
		}

		for(int j=0;j<A.col_size();j++)
		{
			tmp_d = 0.0;
			for (int k = 0; k < sig_num; k++)
			{
				tmp_d += tmp[k] * svdd.V[k][j];
			}
			cout<<setw(12)<<tmp_d<<' ';
		}
		cout<<endl;
	}
	cout<<endl;

	return 0;
}